/*
 * This file is part of Bielefeld Type Library (BTL).
 *
 * Copyright(c) Frederic Siepmann
 * http://opensource.cit-ec.de/projects/btl
 *
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 *
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 *
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 *
 * Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman Köster,
 * 				Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling,
 * 				Denis Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
 */

/*
 * Rotation3D.cpp
 *
 *  Created on: Apr 11, 2011
 *      Author: lziegler
 */

#include "Rotation3D.h"

using namespace xmltio;
using namespace std;

namespace airobots {

const string Rotation3D::labels[3][3] = {{"a00","a01","a02"},
											  {"a10","a11","a12"},
											  {"a20","a21","a22"}};

Rotation3D::Rotation3D() {
	mat.setIdentity();
}

Rotation3D::Rotation3D(double x, double y, double z, double angle,
		AngleUnit angleUnit) {
	setAxisAngle(x, y, z, angle, angleUnit);
}

Rotation3D::Rotation3D(const Eigen::Quaterniond &quaternion) {
	setQuaternion(quaternion);
}

Rotation3D::Rotation3D(double heading, double attitude, double bank,
		AngleUnit angleUnit) {
	setEuler(heading, attitude, bank, angleUnit);
}

Rotation3D::Rotation3D(const Eigen::Vector3d &vec, AngleUnit angleUnit) {
	setEuler(vec, angleUnit);
}

Rotation3D::Rotation3D(const Eigen::Matrix3d &mat) :
		mat(mat) {
}

Rotation3D::~Rotation3D() {
}

void Rotation3D::setMatrix(const Eigen::Matrix3d &mat) {
	this->mat = mat;
}

void Rotation3D::setAxisAngle(double x, double y, double z, double angle,
		AngleUnit angleUnit) {

	Eigen::Vector3d axis(x, y, z);
	setAxisAngle(axis, angle, angleUnit);
}



void Rotation3D::setAxisAngle(const Eigen::AngleAxisd &a) {

	mat = a.toRotationMatrix();

}

void Rotation3D::setQuaternion(const Eigen::Quaterniond &quaternion) {
	Eigen::Quaterniond q(quaternion);
	q.normalize();
	mat = q;
}

void Rotation3D::setEuler(double heading, double attitude, double bank,
		AngleUnit angleUnit) {

	double h = UnitConverter::convert(heading, angleUnit, RADIAN);
	double a = UnitConverter::convert(attitude, angleUnit, RADIAN);
	double b = UnitConverter::convert(bank, angleUnit, RADIAN);

	// Algorithm from here:
	// http://www.euclideanspace.com/maths/geometry/rotations/conversions/eulerToMatrix/index.htm

	double ch = cos(h);
	double sh = sin(h);
	double ca = cos(a);
	double sa = sin(a);
	double cb = cos(b);
	double sb = sin(b);

	mat(0, 0) = ch * ca;
	mat(1, 0) = sh * sb - ch * sa * cb;
	mat(2, 0) = ch * sa * sb + sh * cb;
	mat(0, 1) = sa;
	mat(1, 1) = ca * cb;
	mat(2, 1) = -ca * sb;
	mat(0, 2) = -sh * ca;
	mat(1, 2) = sh * sa * cb + ch * sb;
	mat(2, 2) = -sh * sa * sb + ch * cb;
}

void Rotation3D::setEuler(const Eigen::Vector3d &vec, AngleUnit angleUnit) {

	setEuler((double) vec(0), (double) vec(1), (double) vec(2), angleUnit);
}

Eigen::Vector3d Rotation3D::getEuler(AngleUnit angleUnit) const {

	// Algorithm from here:
	// http://www.euclideanspace.com/maths/geometry/rotations/conversions/matrixToEuler/index.htm

	double heading, attitude, bank;

	if (mat(0, 1) > 0.99999) {
		// singularity at north pole
		heading = atan2((double) mat(2, 0), (double) mat(2, 2));
		attitude = M_PI_2;
		bank = 0;
	} else if (mat(0, 1) < -0.99999) {
		// singularity at south pole
		heading = atan2((double) mat(2, 0), (double) mat(2, 2));
		attitude = -M_PI_2;
		bank = 0;
	} else {
		heading = atan2((double) -mat(0, 2), (double) mat(0, 0));
		bank = atan2((double) -mat(2, 1), (double) mat(1, 1));
		attitude = asin((double) mat(0, 1));
	}

	double h = UnitConverter::convert(heading, RADIAN, angleUnit);
	double a = UnitConverter::convert(attitude, RADIAN, angleUnit);
	double b = UnitConverter::convert(bank, RADIAN, angleUnit);

	return Eigen::Vector3d(h, a, b);
}

Eigen::Matrix3d Rotation3D::getMatrix() const {
	return mat;
}

void Rotation3D::getAxisAngle(Eigen::Vector3d &axis, double &angle,
		AngleUnit angleUnit) const {

	// Algorithm from here:
	// http://www.euclideanspace.com/maths/geometry/rotations/conversions/quaternionToAngle/index.htm

	Eigen::Quaterniond q = getQuaternion();
	if (q.w() > 1)
		q.normalize();
	angle = 2 * acos(q.w());
	double s = sqrt(1 - q.w() * q.w());
	if (s < 0.00001) {
		axis(0) = q.x();
		axis(1) = q.y();
		axis(2) = q.z();
	} else {
		axis(0) = q.x() / s;
		axis(1) = q.y() / s;
		axis(2) = q.z() / s;
	}

	angle = UnitConverter::convert(angle, RADIAN, angleUnit);
}
Eigen::Quaterniond Rotation3D::getQuaternion() const {
	return Eigen::Quaterniond(mat);
}

string Rotation3D::getBaseTag() {
	return "ROTATION3D";
}

string Rotation3D::getTemplate() {
	return "<" + getBaseTag() + "><GENERATOR/>" + Timestamp::getTemplate()
			+ "<MATRIX a00=\"\" a01=\"\" a02=\"\" a10=\"\" a11=\"\" a12=\"\" a20=\"\" a21=\"\" a22=\"\" />" + "</" + getBaseTag() + ">";
}


void Rotation3D::fillInto(Location &loc) const {

	Type::fillInto(loc);
	Location matrix = loc[XPath("MATRIX")];
	for (int i = 0; i < 3; ++i)
		for (int j = 0; j < 3; ++j)
			loc[XPath("MATRIX/@" + labels[i][j])] = (double) mat(i, j);

}

void Rotation3D::fromLocation(const Location &loc, Rotation3D &object) {

	Type::fromLocation(loc, object);
	Eigen::Matrix3d mat;
	try {
		for (int i = 0; i < 3; ++i)
			for (int j = 0; j < 3; ++j)
				mat(i,j) = extract<double>(loc[XPath("MATRIX/@" + labels[i][j])]);
	} catch (ParsingException &e) {
		throw ParsingException("wrong number of matrix entries. Required: 9",
				loc.getDocumentText());
	}

	object.setMatrix(mat);
}

void Rotation3D::sanitizeLocation(Location &loc) const {
	XPath xp(getBaseTag());
	while (XPathIterator i = xp.evaluate(loc)) {
		Location l(*i);
		loc.remove(l);
	}
}

bool Rotation3D::operator==(const Rotation3D &other) const {
	return Type::operator ==(other) && other.mat.isApprox(this->mat, 0.0001);
}

std::ostream& operator<<(std::ostream& o, Rotation3D const& p) {

	return o << "Rotation3D [gen = " << p.getGenerator() << " timestamp = "
			<< p.getTimestamp() << " matrix = " << p.getMatrix() << "]";

}
}
