/*
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * Copyright(c) Frederic Siepmann http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the GNU Lesser General Public
 * License Version 3 (the ``LGPL''), or (at your option) any later version.
 * 
 * Software distributed under the License is distributed on an ``AS IS'' basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the LGPL for the
 * specific language governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this program. If not,
 * go to http://www.gnu.org/licenses/lgpl.html or write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 * 
 * The development of this software was supported by the Excellence Cluster EXC
 * 277 Cognitive Interaction Technology. The Excellence Cluster EXC 277 is a
 * grant of the Deutsche Forschungsgemeinschaft (DFG) in the context of the
 * German Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Robin Schiewer, Sebastian 
 * Meyer zu Borgsen, Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Tristan 
 * Walter, Viktor Losing, Viktor Richter
 */
package de.unibi.airobots.btl.data.grasp;

import nu.xom.Attribute;
import nu.xom.Element;
import nu.xom.ParsingException;
import de.unibi.airobots.btl.Type;
import de.unibi.airobots.btl.units.LengthUnit;
import de.unibi.airobots.btl.units.UnitConverter;

/**
 * This class represents a point in space by all 3 dimensions.
 * 
 * @author lziegler
 */
public class GraspReturnType extends Type {

	/**
	 * Scope of the coordinates.
	 * 
	 * @author semeyerz
	 */
	public enum GraspResult {
		SUCCESS, POSITION_UNREACHABLE, ROBOT_CRASHED, FAIL, COLLISION_HANDLED, NO_RESULT
	};

	// coordinates in millimeters
	private double x;
	private double y;
	private double z;
	private double rating;
	private GraspResult gr = GraspResult.NO_RESULT;

	/**
	 * Creates instance.
	 */
	public GraspReturnType() {
	}

	/**
	 * Creates instance.
	 * 
	 * @param x
	 *            first value
	 * @param y
	 *            second value
	 * @param z
	 *            third value
	 * @param internalUnit
	 *            unit of the values
	 */
	public GraspReturnType(GraspResult gr) {
		this.gr = gr;
	}

	/**
	 * Creates instance.
	 * 
	 * @param x
	 *            first value
	 * @param y
	 *            second value
	 * @param z
	 *            third value
	 * @param unit
	 *            unit of the values
	 * @param scope
	 *            the scope of the coordinates.
	 */
	public GraspReturnType(double x, double y, double z, LengthUnit unit, double rating, GraspResult gr) {
		setX(x, unit);
		setY(y, unit);
		setZ(z, unit);
		this.rating = rating;
		this.gr = gr;
	}

	public double getX(LengthUnit unit) {
		return UnitConverter.convert(x, LengthUnit.MILLIMETER, unit);
	}

	public void setX(double x, LengthUnit unit) {
		this.x = UnitConverter.convert(x, unit, LengthUnit.MILLIMETER);
	}

	public double getY(LengthUnit unit) {
		return UnitConverter.convert(y, LengthUnit.MILLIMETER, unit);
	}

	public void setY(double y, LengthUnit unit) {
		this.y = UnitConverter.convert(y, unit, LengthUnit.MILLIMETER);
	}

	public double getZ(LengthUnit unit) {
		return UnitConverter.convert(z, LengthUnit.MILLIMETER, unit);
	}

	public void setZ(double z, LengthUnit unit) {
		this.z = UnitConverter.convert(z, unit, LengthUnit.MILLIMETER);
	}
	
	public double getRating() {
		return rating;
	}

	public void setRating(double rating) {
		this.rating = rating;
	}

	public GraspResult getGraspResult() {
		return gr;
	}

	public void setGraspResult(GraspResult gr) {
		this.gr = gr;
	}

	public static String getBaseTag() {
		return "GRASPRETURNTYPE";
	}

	/**
	 * Constructs a {@link GraspReturnType} object from a given XOM {@link Element}.
	 * 
	 * @param objectElement
	 *            The XOM {@link Element} to construct an object from.
	 * @return The {@link GraspReturnType} object containing all the information given
	 *         by the {@link Element} object.
	 */
	public static void fromElement(Element objectElement, GraspReturnType object)
			throws ParsingException {

		try {

			double x0 = Double.parseDouble(objectElement.getAttribute("x")
					.getValue());
			double y0 = Double.parseDouble(objectElement.getAttribute("y")
					.getValue());
			double z0 = Double.parseDouble(objectElement.getAttribute("z")
					.getValue());
			double rating0 = Double.parseDouble(objectElement.getAttribute("rating")
					.getValue());

			GraspResult gr0;
			try {
				gr0 = GraspResult.valueOf(objectElement.getAttribute("graspResult")
						.getValue());
			} catch (IllegalArgumentException e) {
				gr0 = GraspResult.NO_RESULT;
			} catch (NullPointerException e) {
				gr0 = GraspResult.NO_RESULT;
			}

			object.setGraspResult(gr0);
			object.setX(x0, LengthUnit.MILLIMETER);
			object.setY(y0, LengthUnit.MILLIMETER);
			object.setZ(z0, LengthUnit.MILLIMETER);
			object.setRating(rating0);

		} catch (NullPointerException ex) {

			// this happens when an element or attribute that is required is
			// not present

			throw new ParsingException("Missing element or attribute "
					+ "in document.", ex);
		} catch (NumberFormatException e) {
			throw new ParsingException("could not parse coordinate values.", e);
		}
	}

	/**
	 * Constructs a {@link GraspReturnType} object from a given XOM {@link Element}.
	 * 
	 * @param objectElement
	 *            The XOM {@link Element} to construct an object from.
	 * @return The {@link GraspReturnType} object containing all the information given
	 *         by the {@link Element} object.
	 */
	public static GraspReturnType fromElement(Element objectElement) 
	throws ParsingException {
		GraspReturnType p = new GraspReturnType();
		fromElement(objectElement, p);
		return p;
	}

	/**
	 * Serializes the {@link GraspReturnType} object into a given XOM {@link Element}.
	 * 
	 * @param parent
	 *            The {@link Element} to serialize the object into. The given
	 *            {@link Element} object should have the base tag defined by
	 *            this class. (see {@link #getBaseTag()})
	 * @see #getBaseTag()
	 */
	public void fillInto(Element parent) {

		parent.addAttribute(new Attribute("x", String
				.valueOf(getX(LengthUnit.MILLIMETER))));
		parent.addAttribute(new Attribute("y", String
				.valueOf(getY(LengthUnit.MILLIMETER))));
		parent.addAttribute(new Attribute("z", String
				.valueOf(getZ(LengthUnit.MILLIMETER))));
		parent.addAttribute(new Attribute("rating", String
				.valueOf(getRating())));
		parent.addAttribute(new Attribute("graspResult", String.valueOf(gr)));
	}

	@Override
	public boolean equals(Object obj) {
		try {
			if (!(obj instanceof GraspReturnType))
				return false;

			GraspReturnType other = (GraspReturnType) obj;

			if (other.getX(LengthUnit.MILLIMETER) != getX(LengthUnit.MILLIMETER))
				return false;
			if (other.getY(LengthUnit.MILLIMETER) != getY(LengthUnit.MILLIMETER))
				return false;
			if (other.getZ(LengthUnit.MILLIMETER) != getZ(LengthUnit.MILLIMETER))
				return false;
			if (other.getRating() != rating)
				return false;
			if (other.getGraspResult() != gr)
				return false;

		} catch (Exception e) {
			return false;
		}
		return true;
	}

	@Override
	public String toString() {
		return "[" + getBaseTag() + " x=" + x + " y=" + y + " z=" + z + " rating=" + rating
				+ " type=" + gr + "]";
	}
}
