/*
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * Copyright(c) Frederic Siepmann http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the GNU Lesser General Public
 * License Version 3 (the ``LGPL''), or (at your option) any later version.
 * 
 * Software distributed under the License is distributed on an ``AS IS'' basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the LGPL for the
 * specific language governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this program. If not,
 * go to http://www.gnu.org/licenses/lgpl.html or write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 * 
 * The development of this software was supported by the Excellence Cluster EXC
 * 277 Cognitive Interaction Technology. The Excellence Cluster EXC 277 is a
 * grant of the Deutsche Forschungsgemeinschaft (DFG) in the context of the
 * German Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Robin Schiewer, Sebastian 
 * Meyer zu Borgsen, Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Tristan 
 * Walter, Viktor Losing, Viktor Richter
 */
package de.unibi.airobots.btl.data.hardwareinfo;

import nu.xom.Attribute;
import nu.xom.Element;
import de.unibi.airobots.btl.Type;
import de.unibi.airobots.btl.data.common.ElementParser;
import de.unibi.airobots.btl.units.*;

/**
 * Representation of camera targets.
 * 
 * @author jwienke
 */
public class CameraTargetData extends Type {

	private double distance = 0;
	private double height = 0;
	private double azimuth = 0;
	private LengthUnit iLU = LengthUnit.MILLIMETER;
	private AngleUnit iAU = AngleUnit.RADIAN;

	private double zoom;

	/**
	 * Constructor.
	 */
	public CameraTargetData() {
		super();
	}

	/**
	 * Constructor.
	 */
	public CameraTargetData(double distance, double height, double azimuth) {
		super();
		this.distance = distance;
		this.height = height;
		this.azimuth = azimuth;
	}

	/**
	 * Returns the distance of the target to focus in laser coordinates in desired length unit.
	 * 
	 * @return distance to focus in laser coordinates
	 */
	public double getDistance(LengthUnit lU) {
		return UnitConverter.convert(distance, iLU, lU);
	}

	/**
	 * Sets the distance of the object to focus in laser coordinates converted incoming 
	 * length unit to mm.
	 * 
	 * @param distance
	 *            distance to focus in mm
	 */
	public void setDistance(double distance,LengthUnit lU) {
		this.distance = UnitConverter.convert(distance, lU, iLU);
	}

	/**
	 * Returns the height of the object to focus in coordinates in desired length unit.
	 * 
	 * @return height of the object to focus in mm
	 */
	public double getHeight(LengthUnit lU) {
		return UnitConverter.convert(height, iLU, lU);
	}

	/**
	 * Sets the height of the object to focus in coordinates, converted incoming 
	 * length unit to mm above floor.
	 * 
	 * @param height
	 *            height to focus in mm above floor
	 */
	public void setHeight(double height, LengthUnit lU) {
		this.height = UnitConverter.convert(height, lU, iLU);
	}

	/**
	 * Returns the azimuth angle of the object to focus in desired angle unit.
	 * 
	 * @return azimuth angle to focus (0 = straight)
	 */
	public double getAzimuth(AngleUnit aU) {
		return UnitConverter.convert(azimuth, iAU, aU);
	}

	/**
	 * Sets the azimuth angle of the object to focus in rad.
	 * 
	 * @param azimuth
	 *            azimuth angle to focus in rad (0 = straight)
	 */
	public void setAzimuth(double azimuth,AngleUnit aU) {
		this.azimuth = UnitConverter.convert(azimuth, aU, iAU);
	}

	/**
	 * Returns the zoom level of the camera as percent of the total zoom.
	 * 
	 * @return zoom [0..1]
	 */
	public double getZoom() {
		assert (zoom >= 0 && zoom <= 1) : "Zoom must be a percent value [0..1].";
		return zoom;
	}

	/**
	 * Sets the zoom as percent of the total zoom of the camera.
	 * 
	 * @param zoom
	 *            desired zoom [0..1]
	 * @throws IllegalArgumentException
	 *             illegal zoom value
	 */
	public void setZoom(double zoom) throws IllegalArgumentException {
		if (zoom < 0 || zoom > 1) {
			throw new IllegalArgumentException(
					"Zoom must be a value in [0..1] but got " + zoom + ".");
		}
		this.zoom = zoom;
	}

	public static String getBaseTag() {
		return "CAMERATARGET";
	}

	public static void fromElement(Element element, CameraTargetData type) {

		Type.fromElement(element, type);

		Element cylindricElement = element.getFirstChildElement("CYLINDRIC");
		type.setDistance(ElementParser.getDoubleAttributeValue(
				cylindricElement, "dist"),LengthUnit.MILLIMETER);
		type.setHeight(ElementParser.getDoubleAttributeValue(cylindricElement,
				"height"),LengthUnit.MILLIMETER);

		Element azimuthElement = element.getFirstChildElement("AZIMUTH");
		type.setAzimuth(ElementParser.getDoubleAttributeValue(azimuthElement,
				"rad"),AngleUnit.RADIAN);

		Element zoomElement = element.getFirstChildElement("ZOOM");
		type.setZoom(ElementParser.getDoubleAttributeValue(zoomElement, "qual"));

	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void fillInto(Element parent) {
		super.fillInto(parent);

		Element cylindricElement = new Element("CYLINDRIC");
		parent.appendChild(cylindricElement);
		cylindricElement.addAttribute(new Attribute("dist", String
				.valueOf(getDistance(LengthUnit.MILLIMETER))));
		cylindricElement.addAttribute(new Attribute("height", String
				.valueOf(getHeight(LengthUnit.MILLIMETER))));

		Element azimuthElement = new Element("AZIMUTH");
		parent.appendChild(azimuthElement);
		azimuthElement.addAttribute(new Attribute("rad", String
				.valueOf(getAzimuth(AngleUnit.RADIAN))));

		Element zoomElement = new Element("ZOOM");
		parent.appendChild(zoomElement);
		zoomElement.addAttribute(new Attribute("qual", String
				.valueOf(getZoom())));

	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toString() {
		return "CameraTargetData [distance = " + distance + ", height = "
				+ height + ", azimuth = " + azimuth + ", zoom = " + zoom + "]";
	}

}
