/*
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * Copyright(c) Frederic Siepmann http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the GNU Lesser General Public
 * License Version 3 (the ``LGPL''), or (at your option) any later version.
 * 
 * Software distributed under the License is distributed on an ``AS IS'' basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the LGPL for the
 * specific language governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this program. If not,
 * go to http://www.gnu.org/licenses/lgpl.html or write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 * 
 * The development of this software was supported by the Excellence Cluster EXC
 * 277 Cognitive Interaction Technology. The Excellence Cluster EXC 277 is a
 * grant of the Deutsche Forschungsgemeinschaft (DFG) in the context of the
 * German Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Robin Schiewer, Sebastian 
 * Meyer zu Borgsen, Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Tristan 
 * Walter, Viktor Losing, Viktor Richter
 */
package de.unibi.airobots.btl.data.map;

import de.unibi.airobots.btl.Type;
import de.unibi.airobots.btl.data.common.ElementParser;
import de.unibi.airobots.btl.data.common.Timestamp;
import de.unibi.airobots.btl.data.geometry.PrecisePolygon;
import de.unibi.airobots.btl.data.navigation.PositionData;
import de.unibi.airobots.btl.units.LengthUnit;
import java.util.Arrays;
import java.util.LinkedList;
import java.util.UUID;
import nu.xom.Attribute;
import nu.xom.Element;
import nu.xom.Elements;
import nu.xom.ParsingException;

/**
 * This type describes an annotation of a map, e.g., rooms or no-go areas. Each
 * annotation is represented by an area (polygon) and has several viewpoints of
 * different categories.
 *
 * <code>
 * <ANNOTATION label="kitchen">
 *      <TIMESTAMP>
 *          <INSERTED value="1334754163505" />
 *          <UPDATED value="1334754163505" />
 *      </TIMESTAMP>
 *      <GENERATOR>unknown</GENERATOR>
 *      <VIEWPOINT label="table1" category="VIEW">
 *          <ROBOTPOSITION>
 *              <TIMESTAMP>
 *                  <INSERTED value="1334589211533" />
 *                  <UPDATED value="1334589211533" />
 *              </TIMESTAMP>
 *              <GENERATOR>unknown</GENERATOR>
 *              <POSITION x="0.0" y="1.0" theta="2.0" ref="world" kind="absolute" />
 *              </ROBOTPOSITION>
 *      </VIEWPOINT>
 *      <VIEWPOINT label="table2" category="VIEW">
 *          <ROBOTPOSITION>
 *              <TIMESTAMP>
 *                  <INSERTED value="1334589211533" />
 *                  <UPDATED value="1334589211533" />
 *              </TIMESTAMP>
 *              <GENERATOR>unknown</GENERATOR>
 *              <POSITION x="0.0" y="1.0" theta="2.0" ref="world" kind="absolute" />
 *          </ROBOTPOSITION>
 *      </VIEWPOINT>
 *      <PRECISEPOLYGON>
 *          <POINT2D x="0.0" y="0.0" scope="GLOBAL" />
 *          <POINT2D x="0.0" y="2000.0" scope="GLOBAL" />
 *          <POINT2D x="2000.0" y="2000.0" scope="GLOBAL" />
 *          <POINT2D x="2000.0" y="0.0" scope="GLOBAL" />
 *      </PRECISEPOLYGON>
 * </ANNOTATION>
 * </code>
 *
 * @author lkettenb
 */
public class Annotation extends Type {

    /**
     * Name/label of this annotation. This label should be unique. Please note
     * that this is not verified until now.
     */
    private String label = UUID.randomUUID().toString();
    /**
     * This polygon describes the area of this annotation.
     */
    private PrecisePolygon polygon = new PrecisePolygon();
    /**
     * List of all viewpoints that belong to this annotation. Every viewpoint
     * should have a unique label. Please note that this is not verified until
     * now.
     */
    private LinkedList<Viewpoint> viewpoints =
            new LinkedList<Viewpoint>();

    /**
     * Default constructor as expected by {@link Type}. You should not use it!
     */
    public Annotation() {
    }

    /**
     * Constructor.
     *
     * @param label Name of this annotation.
     * @param polygon Polygon that describes the area of this annotation.
     * @param viewpoints One or more initial viewpoints that belong to this
     * annotation.
     */
    public Annotation(String label, PrecisePolygon polygon,
            Viewpoint... viewpoints) {
        this.label = label;
        this.polygon = polygon;
        this.viewpoints.addAll(Arrays.asList(viewpoints));
    }

    /**
     * Constructor.
     *
     * @param label Name of this annotation.
     * @param polygon Polygon that describes the area of this annotation.
     * @param viewpoints One or more initial viewpoints that belong to this
     * annotation.
     */
    public Annotation(String label, PrecisePolygon polygon,
            LinkedList<Viewpoint> viewpoints) {
        this.label = label;
        this.polygon = polygon;
        this.viewpoints = viewpoints;
    }

    /**
     * Use this method to access the polygon of this annotation. You may use it
     * to determine whether a point is inside this polygon.
     *
     * @see PrecisePolygon#contains(double, double)
     *
     * @return Polygon of this annotation.
     */
    public PrecisePolygon getPolygon() {
        return polygon;
    }

    /**
     * Returns a list of all viewpoints. Please note that a viewpoint is not
     * necessarily inside of the annotation its polygon/area.
     *
     * @return List of all viewpoints.
     */
    public LinkedList<Viewpoint> getViewpoints() {
        return viewpoints;
    }

    /**
     * Returns nearest viewpoint to given position
     *
     * @param position given position
     * @return nearest viewpoint
     */
    public Viewpoint getNearestViewPoint(PositionData position) {
        double minDist = Double.MAX_VALUE;
        Viewpoint result = null;
        for (Viewpoint vp : viewpoints) {
            double distance = Math.sqrt(Math.pow(vp.getCoordinates().getX(LengthUnit.METER)
                    - position.getX(LengthUnit.METER), 2)
                    + Math.pow(
                    vp.getCoordinates().getY(LengthUnit.METER)
                    - position.getY(LengthUnit.METER), 2));
            if (distance < minDist) {
                minDist = distance;
                result = vp;
            }
        }
        return result;
    }

    /**
     * Label/name of this annotation.
     *
     * @return Label/name of this annotation.
     */
    public String getLabel() {
        return label;
    }

    public void setPolygon(PrecisePolygon polygon) {
        this.polygon = polygon;
    }

    public void setViewpoints(LinkedList<Viewpoint> viewpoints) {
        this.viewpoints = viewpoints;
    }

    public void setLabel(String label) {
        this.label = label;
    }

    /**
     * Getter for the XML base tag used for this (de-)serialization.
     *
     * @return XML base tag.
     */
    public static String getBaseTag() {
        return "ANNOTATION";
    }

    /**
     * Constructs a {@link Annotation} object from a given XOM {@link Element}.
     *
     * @param element The XOM {@link Element} to construct an object from.
     *
     * @return The {@link Annotation} object containing all the information
     * given by the {@link Element} object.
     */
    public static Annotation fromElement(Element element) {
        String tmpLabel = "";
        PrecisePolygon tmpPolygon = new PrecisePolygon();
        LinkedList<Viewpoint> tmpViewpoints =
                new LinkedList<Viewpoint>();

        try {
            if (!element.getLocalName().equals(getBaseTag())) {
                throw new IllegalArgumentException("Parsing Error: Base tag "
                        + "for annotation object must be '"
                        + getBaseTag() + "'");
            }

            tmpLabel = ElementParser.getAttributeValue(element, "label");

            Elements viewpoints = element.getChildElements(Viewpoint.getBaseTag());
            for (int i = 0; i < viewpoints.size(); i++) {
                tmpViewpoints.add(Viewpoint.fromElement(viewpoints.get(i)));
            }

            Element elementPolygon = element.getFirstChildElement(
                    PrecisePolygon.getBaseTag());
            tmpPolygon = PrecisePolygon.fromElement(elementPolygon);
        } catch (ParsingException ex) {
            // This exception is never thrown by PrecisePolygon.fromElement...
        } catch (NullPointerException ex) {
            // this happens when an element or attribute that is required is
            // not present
            throw new IllegalArgumentException("Missing element or attribute "
                    + "in document.", ex);
        }

        return new Annotation(tmpLabel, tmpPolygon, tmpViewpoints);
    }

    /**
     * Constructs a {@link Annotation} object from a given XOM {@link Element}.
     *
     * @param element The XOM {@link Element} to construct an object from.
     * @param annotation The {@link Annotation} object to be filled with all the
     * information given by the {@link Element} object.
     */
    public static void fromElement(Element element, Annotation annotation) {
        Annotation parsedAnnotation = fromElement(element);
        annotation.setLabel(parsedAnnotation.getLabel());
        annotation.setPolygon(parsedAnnotation.getPolygon());
        annotation.setViewpoints(parsedAnnotation.getViewpoints());
    }

    /**
     * Serializes this {@link Annotation} object into a given XOM
     * {@link Element}.
     *
     * @param parent The {@link Element} to serialize the object into. The given
     *            {@link Element} object should have the base tag defined by this class.
     * (see {@link #getBaseTag()})
     * @see #getBaseTag()
     */
    @Override
    public void fillInto(Element parent) {
        sanitizeElement(parent);

        parent.addAttribute(new Attribute("label", label));

        // timestamp
        Element timestampElement = new Element(Timestamp.getBaseTag());
        getTimestamp().fillInto(timestampElement);
        parent.appendChild(timestampElement);

        // generator
        Element generatorElement = new Element("GENERATOR");
        generatorElement.appendChild(getGenerator());
        parent.appendChild(generatorElement);

        for (Viewpoint viewpoint : viewpoints) {
            Element item = new Element(Viewpoint.getBaseTag());
            viewpoint.fillInto(item);
            parent.appendChild(item);
        }

        Element item = new Element(PrecisePolygon.getBaseTag());
        polygon.fillInto(item);
        parent.appendChild(item);
    }

    @Override
    public boolean equals(Object obj) {
        try {
            if (!(obj instanceof Annotation)) {
                return false;
            }

            Annotation other = (Annotation) obj;

            if (!other.getLabel().equals(getLabel())) {
                return false;
            }
            if (!other.getPolygon().equals(getPolygon())) {
                return false;
            }
            if (!other.getViewpoints().equals(getViewpoints())) {
                return false;
            }

        } catch (Exception e) {
            return false;
        }
        return true;
    }

    @Override
    public int hashCode() {
        int hash = 5;
        hash = 97 * hash + (this.label != null ? this.label.hashCode() : 0);
        hash = 97 * hash + (this.polygon != null ? this.polygon.hashCode() : 0);
        hash = 97 * hash + (this.viewpoints != null ? this.viewpoints.hashCode() : 0);
        return hash;
    }
}
