/*
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * Copyright(c) Frederic Siepmann http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the GNU Lesser General Public
 * License Version 3 (the ``LGPL''), or (at your option) any later version.
 * 
 * Software distributed under the License is distributed on an ``AS IS'' basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the LGPL for the
 * specific language governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this program. If not,
 * go to http://www.gnu.org/licenses/lgpl.html or write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 * 
 * The development of this software was supported by the Excellence Cluster EXC
 * 277 Cognitive Interaction Technology. The Excellence Cluster EXC 277 is a
 * grant of the Deutsche Forschungsgemeinschaft (DFG) in the context of the
 * German Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Robin Schiewer, Sebastian 
 * Meyer zu Borgsen, Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Tristan 
 * Walter, Viktor Losing, Viktor Richter
 */
package de.unibi.airobots.btl.data.medicalreport;

import nu.xom.Attribute;
import nu.xom.Element;
import nu.xom.Node;
import nu.xom.Nodes;
import nu.xom.ParsingException;
import de.unibi.airobots.btl.Type;
import de.unibi.airobots.btl.data.common.ElementParser;
import de.unibi.airobots.btl.data.navigation.PositionData;
import de.unibi.airobots.btl.units.*;

/**
 * The medical class holds the informations about the medical report of a person, who BIRON detects.
 * This class is the object, the PersonSensor return. The informations are:
 * <ul>
 * <li>the MedicalData's ID
 * <li>the Person's name from which the mdicaldata is taken, if known to BIRON
 * <li>if the Person is injured 
 * <li>the Person's injury
 * <li>if the doctor is coming today to the person
 * <li>if the the person should get medicine
 * </ul>
 * 
 * @author dnacke 
 * 
 */
public class MedicalData extends Type {

	private int id;
	private String name = "";
	private boolean isInjured;
	private String injury = "";
	private String bodyPart = "";
	private boolean shouldDoctorVisit;
	private boolean medicine;
	private boolean showBodypart;
	private boolean takePic;
	private String medicineType;
	private int height;
	private int weight;

	private PositionData positionData = new PositionData(0, 0, 0, 0,
			LengthUnit.METER, AngleUnit.RADIAN);

	public MedicalData() {}
	
	public void setId(int id) {
		this.id = id;
	}

	public void setName(String name) {
		this.name = name;
	}
	
	public void setInjury(String injury) {
		this.injury = injury;
	}
	
	public void setBodyPart(String bodyPart) {
		this.bodyPart = bodyPart;
	}
	
	public void setShouldDoctorVisit(boolean shouldDoctorVisit) {
		this.shouldDoctorVisit = shouldDoctorVisit;
	}
	
	public void setIsInjured(boolean isInjured) {
		this.isInjured = isInjured;
	}

	public void setMedicine(boolean medicine) {
		this.medicine = medicine;
	}
	
	public void setShowBodypart(boolean showBodypart) {
		this.showBodypart = showBodypart;
	}
	public void setTakePic(boolean takePic) {
		this.takePic = takePic;
	}
	
	public void setMedicineType(String medicineType) {
		this.medicineType = medicineType;
	}
	
	public void setHeight(int height) {
		this.height = height;
	}
	
	public void setWeight(int ẃeight) {
		this.weight = weight;
	}
	/**
	 * A getter for the MedicalData internal ID.
	 * 
	 * @return the MedicalData's internal ID.
	 */
	public int getId() {
		return id;
	}

	/**
	 * A getter for the Person's name in the MedicalData. If the Person is known to BIRON, a name
	 * will be assigned.
	 * 
	 * @return the Person's name, if known.
	 */
	public String getName() {
		return name;
	}

	/**
	 * A getter for the Person's injury in the MedicalData. If the Injury is known to BIRON, a name
	 * will be assigned.
	 * 
	 * @return the Injury's name, if known.
	 */
	public String getInjury() {
		return injury;
	}
	
	public String getBodyPart() {
		return bodyPart;
	}
	public String getMedicineType() {
		return medicineType;
	}
	/**
	 * A getter to check, if the doctor is coming or not.
	 * 
	 * @return whether the Person is facing BIRON.
	 */
	public boolean shouldDoctorVisit() {
		return shouldDoctorVisit;
	}
	
	/**
	 * A getter to check, if the person has an injury or not.
	 * 
	 * @return whether the Person is facing BIRON.
	 */
	public boolean isInjured() {
		return isInjured;
	}
	
	/**
	 * A getter to check, if the person should get medicine or not.
	 * 
	 * @return whether the Person is facing BIRON.
	 */
	public boolean getMedicine() {
		return medicine;
	}
	
	/**
	 * A getter to check, if the person should get medicine or not.
	 * 
	 * @return whether the Person is facing BIRON.
	 */
	public boolean getShowBodypart() {
		return showBodypart;
	}
	
	
	/**
	 * A getter to check, if the person should get medicine or not.
	 * 
	 * @return whether the Person is facing BIRON.
	 */
	public boolean getTakePic() {
		return takePic;
	}
	
	public int getHeight() {
		return height;
	}
	
	public int getWeight() {
		return weight;
	}
	
	/**
	 * A getter for the global position of a person.
	 * 
	 * @deprecated Not implemented. Use the relative position information from
	 *             {@link #getDistance(LengthUnit)} and
	 *             {@link #getAngle(AngleUnit)} instead.
	 * 
	 */
	@Deprecated
	public PositionData getPositionData() {
		return positionData;
	}

	/**
	 * A setter for the global position of a person. This is the postion where
	 * the robot has seen this person last * @param positionData global position
	 * of a person
	 * 
	 * @deprecated Not implemented. Use the relative position information from
	 *             {@link #setDistance(double, LengthUnit)} and
	 *             {@link #setAngle(double, AngleUnit)} instead.
	 */
	@Deprecated
	public void setPositionData(PositionData positionData) {
		this.positionData = positionData;
	}
	
	/**
	 * @return a String with the MedicalData.
	 */
	@Override
	public String toString() {
		StringBuilder strb = new StringBuilder();
		strb.append("#MEDICALDATA# ");
		strb.append("id: " + id + "; ");
		strb.append("name: \"" + name + "\"; ");
		strb.append("injury: " + injury + "; ");
		strb.append("shouldDoctorVisit: " + shouldDoctorVisit + "; ");
		strb.append("isInjured: " + isInjured + "; ");
		strb.append("medicine: " + medicine + "; ");
		strb.append("bodyPart: " + bodyPart + "; ");
		strb.append("showbodyPart: " + showBodypart + "; ");
		strb.append("takePic: " + takePic + "; ");
		strb.append("medicineType: " + medicineType + "; ");
		strb.append("height: " + height + "; ");
		strb.append("weight: " + weight + "; ");
		strb.append(positionData.toString());
		return strb.toString();
	}

	public static String getBaseTag() {
		return "MEDICALDATA";
	}

	public static void fromElement(Element element, MedicalData type) {

		Type.fromElement(element, type);

		Nodes nodes = element.query("*");
		for (int i = 0; i < nodes.size(); i++) {
			Node node = nodes.get(i);
			if (node instanceof Element) {
				Element elem = (Element) node;
				if (elem.getLocalName().equals("STATES")) {
					type.setShouldDoctorVisit(ElementParser.getBooleanAttributeValue(elem,
							"shouldDoctorVisit"));
					type.setIsInjured(ElementParser.getBooleanAttributeValue(elem,
							"isInjured"));
					type.setIsInjured(ElementParser.getBooleanAttributeValue(elem,
							"getMedicine"));
					type.setIsInjured(ElementParser.getBooleanAttributeValue(elem,
							"getTakePic"));
					type.setIsInjured(ElementParser.getBooleanAttributeValue(elem,
							"getShowBodypart"));
				} else if (elem.getLocalName().equals("IDENT")) {
					type.setId(ElementParser.getIntAttributeValue(elem, "id"));
					type.setName(ElementParser.getAttributeValue(elem, "name"));
					type.setInjury(ElementParser.getAttributeValue(elem, "injury"));
					type.setBodyPart(ElementParser.getAttributeValue(elem, "bodyPart"));
					type.setBodyPart(ElementParser.getAttributeValue(elem, "medicineType"));
					type.setId(ElementParser.getIntAttributeValue(elem, "height"));
					type.setId(ElementParser.getIntAttributeValue(elem, "weight"));
				}
			}
		}

		nodes = element.query(PositionData.getBaseTag());
		int nrNodes = nodes.size();
		for (int i = 0; i < nrNodes; i++) {

			Node node = nodes.get(i);
			if (node instanceof Element) {

				Element positionElement = (Element) node;

				PositionData parsed = new PositionData();
				try {
					PositionData.fromElement(positionElement, parsed);
					type.setPositionData(parsed);
				} catch (ParsingException e) {
					type.setPositionData(null);
					e.printStackTrace();
				}

			}

		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void fillInto(Element parent) {
		super.fillInto(parent);

		Element positionDataElement = new Element(PositionData.getBaseTag());
		positionData.fillInto(positionDataElement);
		parent.appendChild(positionDataElement);

		Element statesElement = new Element("STATES");
		parent.appendChild(statesElement);
		statesElement.addAttribute(new Attribute("shouldDoctorVisit",
				(shouldDoctorVisit() ? "YES" : "NO")));
		statesElement.addAttribute(new Attribute("isInjured",
				(isInjured() ? "YES" : "NO")));
		
		statesElement.addAttribute(new Attribute("medicine",
				(getMedicine() ? "YES" : "NO")));
		
		statesElement.addAttribute(new Attribute("showbodyPart",
				(getShowBodypart() ? "YES" : "NO")));
		
		statesElement.addAttribute(new Attribute("takePic",
				(getTakePic() ? "YES" : "NO")));


		Element identElement = new Element("IDENT");
		parent.appendChild(identElement);
		identElement.addAttribute(new Attribute("id", String.valueOf(getId())));
		identElement.addAttribute(new Attribute("name", getName()));
		identElement.addAttribute(new Attribute("injury", getInjury()));
		identElement.addAttribute(new Attribute("bodyPart", getBodyPart()));
		identElement.addAttribute(new Attribute("medicineType", getMedicineType()));
		identElement.addAttribute(new Attribute("height", String.valueOf(getHeight())));
		identElement.addAttribute(new Attribute("weight", String.valueOf(getWeight())));
	}
}
