/*
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * Copyright(c) Frederic Siepmann http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the GNU Lesser General Public
 * License Version 3 (the ``LGPL''), or (at your option) any later version.
 * 
 * Software distributed under the License is distributed on an ``AS IS'' basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the LGPL for the
 * specific language governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this program. If not,
 * go to http://www.gnu.org/licenses/lgpl.html or write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 * 
 * The development of this software was supported by the Excellence Cluster EXC
 * 277 Cognitive Interaction Technology. The Excellence Cluster EXC 277 is a
 * grant of the Deutsche Forschungsgemeinschaft (DFG) in the context of the
 * German Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Robin Schiewer, Sebastian 
 * Meyer zu Borgsen, Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Tristan 
 * Walter, Viktor Losing, Viktor Richter
 */
package de.unibi.airobots.btl.data.person;

import nu.xom.Attribute;
import nu.xom.Element;
import de.unibi.airobots.btl.Type;
import de.unibi.airobots.btl.data.common.ElementParser;
import de.unibi.airobots.btl.data.common.Timestamp;

/**
 * The face class stores information about face hypothesis coming from the face
 * detector.
 * 
 * @author: dschulze <dschulze@techfak.uni-bielefeld.de>
 */
public class Face extends Type {

	private double reliability;

	private int classId;

	private int xCenter;

	private int yCenter;

	private int rectangleX;

	private int rectangleY;

	private int rectangleWidth;

	private int rectangleHeight;

	public Face() {
		setLegacyParsing(true);
	}

	public void setReliability(double reliability) {
		this.reliability = reliability;
	}

	public double getReliability() {
		return reliability;
	}

	public void setClassId(int classId) {
		this.classId = classId;
	}

	public int getClassId() {
		return classId;

	}

	public void setXCenter(int xCenter) {
		this.xCenter = xCenter;
	}

	public int getXCenter() {
		return xCenter;
	}

	public void setYCenter(int yCenter) {
		this.yCenter = yCenter;
	}

	public int getYCenter() {
		return yCenter;
	}

	public void setRectangleX(int rectangleX) {
		this.rectangleX = rectangleX;
	}

	public int getRectangleX() {
		return rectangleX;
	}

	public void setRectangleY(int rectangleY) {
		this.rectangleY = rectangleY;
	}

	public int getRectangleY() {
		return rectangleY;
	}

	public void setRectangleWidth(int width) {
		this.rectangleWidth = width;
	}

	public int getRectangleWidth() {
		return rectangleWidth;
	}

	public void setRectangleHeight(int height) {
		this.rectangleHeight = height;
	}

	public int getRectangleHeight() {
		return rectangleHeight;
	}

	public static String getBaseTag() {
		return "OBJECT";
	}

	/** @{inheritDoc */
	@Override
	public void fillInto(Element parent) {

		Element hypothesisElement = new Element("HYPOTHESIS");

		// Add generator information.
		Element generatorElement = new Element("GENERATOR");
		generatorElement.appendChild(getGenerator());
		hypothesisElement.appendChild(generatorElement);

		// Add timestamp information.
		Element timestampsElement = new Element("TIMESTAMPS");
		Element createdElement = new Element("CREATED");
		long time = getTimestamp().getCreated().getTime();
		createdElement
				.addAttribute(new Attribute("value", String.valueOf(time)));
		timestampsElement.appendChild(createdElement);
		Element updatedElement = new Element("UPDATED");
		time = getTimestamp().getUpdated().getTime();
		updatedElement
				.addAttribute(new Attribute("value", String.valueOf(time)));
		timestampsElement.appendChild(updatedElement);
		hypothesisElement.appendChild(timestampsElement);

		// getTimestamp().fillInto(timestampsElement);
		// hypothesisElement.appendChild(timestampsElement);

		// Add rating information.
		Element ratingElement = new Element("RATING");
		Element reliabilityElement = new Element("RELIABILITY");
		reliabilityElement.addAttribute(new Attribute("value", String
				.valueOf(getReliability())));
		ratingElement.appendChild(reliabilityElement);
		hypothesisElement.appendChild(ratingElement);
		parent.appendChild(hypothesisElement);

		// Add class id information.
		Element classElement = new Element("CLASS");
		classElement.addAttribute(new Attribute("id", String
				.valueOf(getClassId())));
		classElement.appendChild("unrecognized class");
		parent.appendChild(classElement);

		// Add region information.
		Element regionElement = new Element("REGION");
		Element centerElement = new Element("CENTER");
		centerElement.addAttribute(new Attribute("x", String
				.valueOf(getXCenter())));
		centerElement.addAttribute(new Attribute("y", String
				.valueOf(getYCenter())));
		regionElement.appendChild(centerElement);
		Element rectangleElement = new Element("RECTANGLE");
		rectangleElement.addAttribute(new Attribute("x", String
				.valueOf(getRectangleX())));
		rectangleElement.addAttribute(new Attribute("y", String
				.valueOf(getRectangleY())));
		rectangleElement.addAttribute(new Attribute("w", String
				.valueOf(getRectangleWidth())));
		rectangleElement.addAttribute(new Attribute("h", String
				.valueOf(getRectangleHeight())));
		regionElement.appendChild(rectangleElement);
		parent.appendChild(regionElement);

	}

	/** @{inheritDoc */
	public static void fromElement(Element element, Face face) {

		Element hypothesisElement = element.getFirstChildElement("HYPOTHESIS");

		// Retrieve timestamp information.
		Element timestampsElement = hypothesisElement
				.getFirstChildElement("TIMESTAMPS");
		Element createdElement = timestampsElement
				.getFirstChildElement("CREATED");
		long creationTime = Long.parseLong(createdElement
				.getAttributeValue("value"));
		Element updatedElement = timestampsElement
				.getFirstChildElement("UPDATED");
		long updatedTime = Long.parseLong(updatedElement
				.getAttributeValue("value"));
		face.setTimestamp(new Timestamp(creationTime, updatedTime));

		System.out.println(element.toXML());
		System.out.println("Parsed times: " + creationTime + " " + updatedTime);

		// Retrieve generator information.
		// Element generatorElement =
		// hypothesisElement.getFirstChildElement("GENERATOR");

		// Retrieve rating information.
		Element ratingElement = hypothesisElement
				.getFirstChildElement("RATING");
		Element reliabilityElement = ratingElement
				.getFirstChildElement("RELIABILITY");
		face.setReliability(ElementParser.getDoubleAttributeValue(
				reliabilityElement, "value"));

		// Retrieve class id value.
		Element classIdElement = element.getFirstChildElement("CLASS");
		face.setClassId(ElementParser
				.getIntAttributeValue(classIdElement, "id"));

		// Retrieve region values.
		Element regionElement = element.getFirstChildElement("REGION");
		Element centerElement = regionElement.getFirstChildElement("CENTER");
		face.setXCenter(ElementParser.getIntAttributeValue(centerElement, "x"));
		face.setYCenter(ElementParser.getIntAttributeValue(centerElement, "y"));
		Element rectangleElement = regionElement
				.getFirstChildElement("RECTANGLE");
		face.setRectangleX(ElementParser.getIntAttributeValue(rectangleElement,
				"x"));
		face.setRectangleY(ElementParser.getIntAttributeValue(rectangleElement,
				"y"));
		face.setRectangleWidth(ElementParser.getIntAttributeValue(
				rectangleElement, "w"));
		face.setRectangleHeight(ElementParser.getIntAttributeValue(
				rectangleElement, "h"));

	}

}
