/*
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * Copyright(c) Frederic Siepmann http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the GNU Lesser General Public
 * License Version 3 (the ``LGPL''), or (at your option) any later version.
 * 
 * Software distributed under the License is distributed on an ``AS IS'' basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the LGPL for the
 * specific language governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this program. If not,
 * go to http://www.gnu.org/licenses/lgpl.html or write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 * 
 * The development of this software was supported by the Excellence Cluster EXC
 * 277 Cognitive Interaction Technology. The Excellence Cluster EXC 277 is a
 * grant of the Deutsche Forschungsgemeinschaft (DFG) in the context of the
 * German Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Robin Schiewer, Sebastian 
 * Meyer zu Borgsen, Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Tristan 
 * Walter, Viktor Losing, Viktor Richter
 */
package de.unibi.airobots.btl.data.status;

import nu.xom.Element;
import nu.xom.ParsingException;

import org.apache.log4j.Logger;

import de.unibi.airobots.btl.Type;
import de.unibi.airobots.btl.data.common.ElementParser;
import de.unibi.airobots.btl.data.common.MicroTimestamp;
import de.unibi.airobots.btl.data.common.Ticked;
import de.unibi.airobots.btl.data.navigation.SpeedData;

/**
 * This class is used to store data from the navigation component. It holds a
 * boolean value describing whether the robot is stalled or not and the time of
 * the sensor values as a timestamp.
 * 
 * @author lziegler
 */
public class StallData extends Type implements Ticked {

	private static Logger logger = Logger.getLogger(SpeedData.class);

	private boolean stalled = false;

	/**
	 * Getter for the stalled value.
	 * 
	 * @return The value describing if the robot is stalled.
	 */
	public boolean isStalled() {
		return stalled;
	}

	/**
	 * Setter for the stalled value.
	 * 
	 * @param stalled
	 *            The value describing if the robot is stalled.
	 */
	public void setStalled(boolean stalled) {
		this.stalled = stalled;
	}

	private MicroTimestamp readTime = new MicroTimestamp();

	/**
	 * Creates a new instance.
	 */
	public StallData() {
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public MicroTimestamp getReadTime() {
		return readTime;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public long getReadTimeSeconds() {
		return readTime.getSeconds();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public long getReadTimeMicroSeconds() {
		return readTime.getMicroSeconds();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void setReadTime(MicroTimestamp readTime) {
		this.readTime = readTime;

	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toString() {
		StringBuilder strb = new StringBuilder();
		strb.append("#STALLED# ");
		strb.append("stalled: " + Boolean.toString(stalled) + "; ");
		return strb.toString();
	}

	/**
	 * Getter for the base tag.
	 * 
	 * @return the base tag.
	 */
	public static String getBaseTag() {
		return "STALL";
	}

	/**
	 * Parses xml in form of a {@link Element} object into a given instance of
	 * {@link StallData}.
	 * 
	 * @param element
	 *            The object containing the xml.
	 * @param type
	 *            The object to put values into.
	 * @throws ParsingException
	 *             Is thrown when parsing failed.
	 */
	public static void fromElement(Element element, StallData type)
			throws ParsingException {

		logger.debug("fromElement called");

		Type.fromElement(element, type);

		Element stallElement = element.getFirstChildElement("stalled");
		String stallString = ElementParser.getTagValue(stallElement);
		boolean stalled = false;
		stalled = Boolean.parseBoolean(stallString);
		type.setStalled(stalled);

		Element timeElement = element.getFirstChildElement(MicroTimestamp
				.getBaseTag());
		MicroTimestamp readTime = new MicroTimestamp();
		MicroTimestamp.fromElement(timeElement, readTime);
		type.setReadTime(readTime);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void fillInto(Element parent) {

		super.fillInto(parent);

		Element readTimeElement = new Element(MicroTimestamp.getBaseTag());
		parent.appendChild(readTimeElement);
		readTime.fillInto(readTimeElement);

		Element stallElement = new Element("stalled");
		parent.appendChild(stallElement);
		stallElement.appendChild(Boolean.toString(stalled));

	}
}
