/*
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * Copyright(c) Frederic Siepmann http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the GNU Lesser General Public
 * License Version 3 (the ``LGPL''), or (at your option) any later version.
 * 
 * Software distributed under the License is distributed on an ``AS IS'' basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the LGPL for the
 * specific language governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this program. If not,
 * go to http://www.gnu.org/licenses/lgpl.html or write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 * 
 * The development of this software was supported by the Excellence Cluster EXC
 * 277 Cognitive Interaction Technology. The Excellence Cluster EXC 277 is a
 * grant of the Deutsche Forschungsgemeinschaft (DFG) in the context of the
 * German Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Robin Schiewer, Sebastian 
 * Meyer zu Borgsen, Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Tristan 
 * Walter, Viktor Losing, Viktor Richter
 */
package de.unibi.airobots.btl.data.vision2d;

import de.unibi.airobots.btl.Type;
import nu.xom.Element;

/**
 * Representation for the image metadata format used by iceWings XCFImageExport.
 * Understands XCF_CONTAINER format (with 5-byte header) and STD format (planar,
 * no header) so far. It is only tested with 3-plane RGB and YUV images. In
 * 4-plane images, the alpha plane is ignored.
 * 
 * @author Ingo Luetkebohle <iluetkeb@techfak.uni-bielefeld.de>
 * @author jwienke
 */
public class ImageData extends Type {

	private static final String IMAGE_URI_ATTRIBUTE = "uri";
	private static final String IMAGE_ELEMENT = "IMAGE";
	private static final String PROPERTIES_ELEMENT = "PROPERTIES";
	private static final String WIDTH_ATTRIBUTE = "width";
	private static final String HEIGHT_ATTRIBUTE = "height";
	private static final String COLORSPACE_ATTRIBUTE = "colorspace";

	private String uri;
	private int width;
	private int height;
	private String colorspace;

	/**
	 * Cosntructor.
	 */
	public ImageData() {
		setLegacyParsing(true);
	}

	public static String getBaseTag() {
		return "IMAGESET";
	}

	public static void fromElement(Element element, ImageData imageData) {

		try {
			Element imageElement = element.getFirstChildElement(IMAGE_ELEMENT);
			Type.fromElement(imageElement, imageData);
			imageData.uri = imageElement.getAttributeValue(IMAGE_URI_ATTRIBUTE);

			// parse rest of metadata
			Element props = imageElement
					.getFirstChildElement(PROPERTIES_ELEMENT);
			imageData.width = Integer.parseInt(props
					.getAttributeValue(WIDTH_ATTRIBUTE));
			imageData.height = Integer.parseInt(props
					.getAttributeValue(HEIGHT_ATTRIBUTE));
			imageData.colorspace = props
					.getAttributeValue(COLORSPACE_ATTRIBUTE).trim();

		} catch (NullPointerException ex) {
			// this happens when an element or attribute that is required is
			// not present
			throw new IllegalArgumentException("Missing element or attribute "
					+ "in document.", ex);
		}
	}

	public int getWidth() {
		return width;
	}

	public void setWidth(int width) {
		this.width = width;
	}

	public String getUri() {
		return uri;
	}

	public int getHeight() {
		return height;
	}

	public String getColorspace() {
		return colorspace;
	}

	public void setUri(String uri) {
		this.uri = uri;
	}

	public void setHeight(int height) {
		this.height = height;
	}

	public void setColorspace(String colorspace) {
		this.colorspace = colorspace;
	}
}
