/*
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * Copyright(c) Frederic Siepmann http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the GNU Lesser General Public
 * License Version 3 (the ``LGPL''), or (at your option) any later version.
 * 
 * Software distributed under the License is distributed on an ``AS IS'' basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the LGPL for the
 * specific language governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this program. If not,
 * go to http://www.gnu.org/licenses/lgpl.html or write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 * 
 * The development of this software was supported by the Excellence Cluster EXC
 * 277 Cognitive Interaction Technology. The Excellence Cluster EXC 277 is a
 * grant of the Deutsche Forschungsgemeinschaft (DFG) in the context of the
 * German Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Robin Schiewer, Sebastian 
 * Meyer zu Borgsen, Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Tristan 
 * Walter, Viktor Losing, Viktor Richter
 */
package de.unibi.airobots.btl.data.vision3d;

import de.unibi.airobots.btl.Type;
import nu.xom.Element;

/**
 * Representation for the image metadata format used by Kinectserver.
 * 
 * @author vrichter
 */
public class KinectImageData extends Type {

	public enum ImageType {
		RGB, DEPTH, REALDEPTH
	}

	private static final String IMAGE_URI_ATTRIBUTE = "uri";
	private static final String IMAGE_ELEMENT = "IMAGE";
	private static final String PROPERTIES_ELEMENT = "PROPERTIES";

	private static final String WIDTH_ATTRIBUTE = "width";
	private static final String HEIGHT_ATTRIBUTE = "height";
	private static final String COLORSPACE_ATTRIBUTE = "colorspace";
	private static final String CHANNELS_ATTRIBUTE = "channels";
	private static final String DEPTH_ATTRIBUTE = "depth";
	private static final String SUBSAMPLING_ATTRIBUTE = "subsampling";

	private static final String RGB_IMAGE_DEPTH_ATTRIBUTE = "depth8u";
	private static final String DEPTH_IMAGE_DEPTH_ATTRIBUTE = "depth16u";
	private static final String REAL_DEPTH_IMAGE_DEPTH_ATTRIBUTE = "depth32f";

	private String uri;
	private int width;
	private int height;
	private String colorspace;
	private int channels;
	private String depth;
	private int subsampling;
	private ImageType type;

	/**
	 * Cosntructor.
	 */
	public KinectImageData() {
		setLegacyParsing(true);
	}

	public static String getBaseTag() {
		return "IMAGESET";
	}

	public static void fromElement(Element element, KinectImageData imageData) {

		try {
			Element imageElement = element.getFirstChildElement(IMAGE_ELEMENT);

			// TODO: what does this do in ImageData? Here it is causing
			// exceptions.
			// Type.fromElement(imageElement, imageData);

			imageData.uri = imageElement.getAttributeValue(IMAGE_URI_ATTRIBUTE);

			// parse rest of metadata
			Element props = imageElement
					.getFirstChildElement(PROPERTIES_ELEMENT);
			imageData.width = Integer.parseInt(props
					.getAttributeValue(WIDTH_ATTRIBUTE));
			imageData.height = Integer.parseInt(props
					.getAttributeValue(HEIGHT_ATTRIBUTE));
			imageData.colorspace = props
					.getAttributeValue(COLORSPACE_ATTRIBUTE).trim();
			imageData.setChannels(Integer.parseInt(props
					.getAttributeValue(CHANNELS_ATTRIBUTE)));
			imageData.setDepth(props.getAttributeValue(DEPTH_ATTRIBUTE).trim());
			imageData.setSubsampling(Integer.parseInt(props
					.getAttributeValue(SUBSAMPLING_ATTRIBUTE)));

			imageData.setType();

		} catch (NullPointerException ex) {
			// this happens when an element or attribute that is required is
			// not present
			throw new IllegalArgumentException("Missing element or attribute "
					+ "in document.", ex);
		}
	}

	private void setType() throws IllegalArgumentException {
		if (this.depth.equals(RGB_IMAGE_DEPTH_ATTRIBUTE)) {
			this.type = ImageType.RGB;
		} else if (this.depth.equals(DEPTH_IMAGE_DEPTH_ATTRIBUTE)) {
			this.type = ImageType.DEPTH;
		} else if (this.depth.equals(REAL_DEPTH_IMAGE_DEPTH_ATTRIBUTE)) {
			this.type = ImageType.REALDEPTH;
		} else {
			throw new IllegalArgumentException("Image depth attribute is '"
					+ this.depth + "' but should be '"
					+ RGB_IMAGE_DEPTH_ATTRIBUTE + "', '"
					+ DEPTH_IMAGE_DEPTH_ATTRIBUTE + "' or '"
					+ REAL_DEPTH_IMAGE_DEPTH_ATTRIBUTE + "'.");
		}
	}

	public ImageType getType() {
		return this.type;
	}

	public int getWidth() {
		return width;
	}

	public void setWidth(int width) {
		this.width = width;
	}

	public String getUri() {
		return uri;
	}

	public int getHeight() {
		return height;
	}

	public String getColorspace() {
		return colorspace;
	}

	public void setUri(String uri) {
		this.uri = uri;
	}

	public void setHeight(int height) {
		this.height = height;
	}

	public void setColorspace(String colorspace) {
		this.colorspace = colorspace;
	}

	public void setChannels(int channels) {
		this.channels = channels;
	}

	public int getChannels() {
		return channels;
	}

	public void setDepth(String depth) {
		this.depth = depth;
	}

	public String getDepth() {
		return depth;
	}

	public void setSubsampling(int subsampling) {
		this.subsampling = subsampling;
	}

	public int getSubsampling() {
		return subsampling;
	}
}
