/*
* This file is part of Bielefeld Type Library (BTL).
*
* Copyright(c) Frederic Siepmann
* http://opensource.cit-ec.de/projects/btl
*
* This file may be licensed under the terms of of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*
* Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman Köster,
* 				Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling,
* 				Denis Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
*/

#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>

#include <iostream>

#include "btl/List.h"
#include "btl/data/person/Face.h"
#include "btl/data/person/Legs.h"
#include "btl/units/AngleUnit.h"
#include "utils.h"

using namespace airobots;
using namespace std;
using namespace xmltio;

static const double tolerance = 0.0001;

BOOST_AUTO_TEST_SUITE(ListTest)

BOOST_AUTO_TEST_CASE(testSelfCompatibility) {

	try {

		// ======================================================================
		// Test with faces.
		// ======================================================================

		List<Face> original;
		original.setGenerator("test");
		original.setTimestamp(Timestamp(123, 456));

		boost::shared_ptr<Face> face = boost::shared_ptr<Face>(new Face);
		face->setGenerator("facesensor");
		face->setAngle(0.34,RADIAN);
		face->setDistance(10.015,MILLIMETER);
		face->setHeight(-23.567,MILLIMETER);
		original.push_back(face);

		boost::shared_ptr<Face> face2 = boost::shared_ptr<Face>(new Face);
		face2->setGenerator("facesensor2");
		face2->setAngle(10.34,RADIAN);
		face2->setDistance(110.015,MILLIMETER);
		face2->setHeight(-123.567,MILLIMETER);
		original.push_back(face2);

		Location originalLoc(List<Face>::getTemplate(), List<Face>::getBaseTag());
		originalLoc = original;

		List<Face> parsed;
		List<Face>::fromLocation(originalLoc, parsed);

		BOOST_REQUIRE_EQUAL(parsed.getGenerator(), original.getGenerator());
		BOOST_REQUIRE_EQUAL(parsed.getTimestamp(), original.getTimestamp());
		BOOST_REQUIRE_EQUAL(parsed.size(), original.size());
		BOOST_REQUIRE_EQUAL(*(parsed[0]), *(original[0]));
		BOOST_REQUIRE_EQUAL(*(parsed[1]), *(original[1]));

		// ======================================================================
		// Test with legs.
		// ======================================================================

		List<Legs> legsOriginal;
		legsOriginal.setGenerator("test");
		legsOriginal.setTimestamp(Timestamp(123, 456));

		boost::shared_ptr<Legs> legs = boost::shared_ptr<Legs>(new Legs);
		legs->setAngle(21312.78,DEGREE);
		legs->setDistance(-1.023,METER);
		legs->setAngleVariance(0.45,DEGREE);
		legs->setDistanceVariance(1.34,METER);
		legs->setNumInvolved(2134);
		legs->setInvolved1(4543);
		legs->setInvolved2(1223);
		legs->setPair(true);

		legsOriginal.push_back(legs);

		boost::shared_ptr<Legs> legs2 = boost::shared_ptr<Legs>(new Legs);
		legs2->setAngle(1234.78,DEGREE);
		legs2->setDistance(-3.023,METER);
		legs2->setAngleVariance(1.45,DEGREE);
		legs2->setDistanceVariance(0.34,METER);
		legs2->setNumInvolved(234);
		legs2->setInvolved1(453);
		legs2->setInvolved2(123);
		legs2->setPair(false);

		legsOriginal.push_back(legs2);

		Location legsOriginalLoc(List<Legs>::getTemplate(), List<Legs>::getBaseTag());
		legsOriginalLoc = legsOriginal;

		List<Legs> legsParsed;
		List<Legs>::fromLocation(legsOriginalLoc, legsParsed);

		BOOST_REQUIRE_EQUAL(legsParsed.getGenerator(), legsOriginal.getGenerator());
		BOOST_REQUIRE_EQUAL(legsParsed.getTimestamp(), legsOriginal.getTimestamp());
		BOOST_REQUIRE_EQUAL(legsParsed.size(), legsOriginal.size());
		BOOST_REQUIRE_EQUAL(*(legsParsed[0]), *(legsOriginal[0]));
		BOOST_REQUIRE_EQUAL(*(legsParsed[1]), *(legsOriginal[1]));

	} catch (std::exception & e) {
		BOOST_FAIL(string("Caught exception: ") + e.what());
	}

}

/*BOOST_AUTO_TEST_CASE(testFileCompatibility) {

 Location fileContents(readFile("List.xml"), List::getBaseTag());
 List parsed = extract<List>(fileContents);

 BOOST_REQUIRE_EQUAL("test", parsed.getGenerator());
 BOOST_REQUIRE_EQUAL(12345, parsed.getTimestamp().getInserted());
 BOOST_REQUIRE_EQUAL(67890, parsed.getTimestamp().getUpdated());
 BOOST_REQUIRE_CLOSE((float) 0.45, parsed.getAngle(), tolerance);
 BOOST_REQUIRE_CLOSE((float) 77.23, parsed.getDistance(), tolerance);

 }*/

BOOST_AUTO_TEST_CASE(testGenericUsage) {

	try {

		List<Face> faces;
		boost::shared_ptr<Face> face = boost::shared_ptr<Face>(new Face);
		face->setGenerator("facesensor");
		face->setAngle(0.34,RADIAN);
		face->setDistance(10.015,MILLIMETER);
		face->setHeight(-23.567,MILLIMETER);
		faces.push_back(face);

		boost::shared_ptr<Face> face2 = boost::shared_ptr<Face>(new Face);
		face2->setGenerator("facesensor2");
		face2->setAngle(10.34,RADIAN);
		face2->setDistance(110.015,MILLIMETER);
		face2->setHeight(-123.567,MILLIMETER);
		faces.push_back(face2);

		List<Type> list = faces;

		BOOST_REQUIRE_EQUAL(faces.getGenerator(), list.getGenerator());
		BOOST_REQUIRE_EQUAL(faces.getTimestamp(), list.getTimestamp());
		BOOST_REQUIRE_EQUAL(faces.size(), list.size());

		const vector<boost::shared_ptr<Face> > convertedFaces = list.getElementsAsType<Face>();
		BOOST_REQUIRE_EQUAL(list.size(), convertedFaces.size());
		BOOST_REQUIRE_EQUAL(*face, *(convertedFaces[0]));
		BOOST_REQUIRE_EQUAL(*face2, *(convertedFaces[1]));

		BOOST_REQUIRE_THROW(list.getElementsAsType<Legs>(), BadSharedPtrCast);

		List<Face> faceList = list;
		BOOST_REQUIRE_THROW(List<Legs> legList = list, BadSharedPtrCast);

	} catch (std::exception & e) {
		BOOST_FAIL(string(e.what()));
	}

}

BOOST_AUTO_TEST_CASE(testLocationReuse) {

	const std::string attributeName = "testattribute";
	const std::string elementName = "ADDITIONAL";

	List<Face> original;
	original.setGenerator("test");
	original.setTimestamp(Timestamp(123, 456));

	boost::shared_ptr<Face> face = boost::shared_ptr<Face>(new Face);
	face->setGenerator("facesensor");
	face->setAngle(0.34,RADIAN);
	face->setDistance(10.015,MILLIMETER);
	face->setHeight(-23.567,MILLIMETER);
	original.push_back(face);

	boost::shared_ptr<Face> face2 = boost::shared_ptr<Face>(new Face);
	face2->setGenerator("facesensor2");
	face2->setAngle(10.34,RADIAN);
	face2->setDistance(110.015,MILLIMETER);
	face2->setHeight(-123.567,MILLIMETER);
	original.push_back(face2);

	Location originalLoc(List<Face>::getTemplate(), List<Face>::getBaseTag());
	originalLoc = original;

	originalLoc[attributeName.c_str()] = 23;
	Location additionalElement("<" + elementName + "/>", elementName);
	originalLoc.add(additionalElement);

	// do some parsing and re-serialization into the old location
	List<Face> parsedFirstTime;
	List<Face>::fromLocation(originalLoc, parsedFirstTime);
	originalLoc = parsedFirstTime;
	List<Face> parsedSecondTime;
	List<Face>::fromLocation(originalLoc, parsedSecondTime);

	// ensure that the parsed data is still the same and additional elements in
	// the xml still exist
	BOOST_REQUIRE_EQUAL(original.size(), parsedSecondTime.size());
	originalLoc[XPath("@" + attributeName)];
	originalLoc[XPath(elementName)];

}

BOOST_AUTO_TEST_SUITE_END()
