/*
* This file is part of Bielefeld Type Library (BTL).
*
* Copyright(c) Frederic Siepmann
* http://opensource.cit-ec.de/projects/btl
*
* This file may be licensed under the terms of of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*
* Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman Köster,
* 				Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling,
* 				Denis Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
*/

#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>
#include "boost/random.hpp"
#include "boost/generator_iterator.hpp"

#include <iostream>

#include "btl/data/navigation/ROSCostmapConfiguration.h"
#include "../../utils.h"

using namespace airobots;
using namespace std;
using namespace xmltio;

static const double TOLERANCE = 0.0001;

BOOST_AUTO_TEST_SUITE(ROSCostmapConfigurationTest)
        
BOOST_AUTO_TEST_CASE(testSelfCompatibility) {

        typedef boost::mt19937 RNGType;
        RNGType rng(time(0)); 
        boost::uniform_real<> one_to_fifty( 0.0, 50.0 );
        boost::variate_generator< RNGType, boost::uniform_real<> >
                    rnd(rng, one_to_fifty);
    
	try {

            ROSCostmapConfiguration original = ROSCostmapConfiguration();
            original.setGenerator("ROSCostmapConfigTest");
            original.setTimestamp(Timestamp(123, 456));
            original.setCostScalingFactor(rnd());
            original.setFootprint("[[-1.0,1.0],[1.0,1.0],[1.0,-1.0],[-1.0,-1.0]]");
            original.setHeight(static_cast<int>(rnd()));
            original.setInflationRadius(rnd());
            original.setIsGlobalCostmap(rnd() > rnd());
            original.setLethalCostThreshold(static_cast<int>(rnd()));
            original.setMapTopic("map");
            original.setMapType("costmap");
            original.setMarkThreshold(static_cast<int>(rnd()));
            original.setMaxObstacleHeight(rnd());
            original.setMaxObstacleRange(rnd());
            original.setOriginX(rnd());
            original.setOriginY(rnd());
            original.setPublishFrequency(rnd());
            original.setPublishVoxelMap(rnd() > rnd());
            original.setRaytraceRange(rnd());
            original.setResolution(rnd());
            original.setRestoreDefaults(rnd() > rnd());
            original.setRobotRadius(rnd());
            original.setRollingWindow(rnd() > rnd());
            original.setStaticMap(rnd() > rnd());
            original.setTrackUnknownSpace(rnd() > rnd());
            original.setTransformTolerance(rnd());
            original.setUnknownCostValue(static_cast<int>(rnd()));
            original.setUnknownThreshold(static_cast<int>(rnd()));
            original.setUpdateFrequency(rnd());
            original.setWidth(static_cast<int>(rnd()));
            original.setzOrigin(rnd());
            original.setzResolution(rnd());
            original.setzVoxels(static_cast<int>(rnd()));
            
          
            Location originalLoc(
                ROSCostmapConfiguration::getTemplate(), 
                ROSCostmapConfiguration::getBaseTag());
            
            originalLoc = original;

            ROSCostmapConfiguration parsed = extract<ROSCostmapConfiguration>(originalLoc);

            BOOST_REQUIRE_EQUAL(parsed.getGenerator(), original.getGenerator());
            BOOST_REQUIRE_EQUAL(parsed.getTimestamp(), original.getTimestamp());
            BOOST_REQUIRE_CLOSE(parsed.getCostScalingFactor(), original.getCostScalingFactor(), TOLERANCE);
            BOOST_REQUIRE_EQUAL(parsed.getFootprint(), original.getFootprint());
            BOOST_REQUIRE_EQUAL(parsed.getHeight(), original.getHeight());
            BOOST_REQUIRE_CLOSE(parsed.getInflationRadius(), original.getInflationRadius(), TOLERANCE);
            BOOST_REQUIRE_EQUAL(parsed.isIsGlobalCostmap(), original.isIsGlobalCostmap());
            BOOST_REQUIRE_EQUAL(parsed.getLethalCostThreshold(), original.getLethalCostThreshold());
            BOOST_REQUIRE_EQUAL(parsed.getMapTopic(), original.getMapTopic());
            BOOST_REQUIRE_EQUAL(parsed.getMapType(), original.getMapType());
            BOOST_REQUIRE_EQUAL(parsed.getMarkThreshold(), original.getMarkThreshold());
            BOOST_REQUIRE_CLOSE(parsed.getMaxObstacleHeight(), original.getMaxObstacleHeight(), TOLERANCE);
            BOOST_REQUIRE_CLOSE(parsed.getMaxObstacleRange(), original.getMaxObstacleRange(), TOLERANCE);
            BOOST_REQUIRE_CLOSE(parsed.getOriginX(), original.getOriginX(), TOLERANCE);
            BOOST_REQUIRE_CLOSE(parsed.getOriginY(), original.getOriginY(), TOLERANCE);
            BOOST_REQUIRE_CLOSE(parsed.getPublishFrequency(), original.getPublishFrequency(), TOLERANCE);
            BOOST_REQUIRE_EQUAL(parsed.isPublishVoxelMap(), original.isPublishVoxelMap());
            BOOST_REQUIRE_CLOSE(parsed.getRaytraceRange(), original.getRaytraceRange(), TOLERANCE);
            BOOST_REQUIRE_CLOSE(parsed.getResolution(), original.getResolution(), TOLERANCE);
            BOOST_REQUIRE_EQUAL(parsed.isRestoreDefaults(), original.isRestoreDefaults());
            BOOST_REQUIRE_CLOSE(parsed.getRobotRadius(), original.getRobotRadius(), TOLERANCE);
            BOOST_REQUIRE_EQUAL(parsed.isRollingWindow(), original.isRollingWindow());
            BOOST_REQUIRE_EQUAL(parsed.isStaticMap(), original.isStaticMap());
            BOOST_REQUIRE_EQUAL(parsed.isTrackUnknownSpace(), original.isTrackUnknownSpace());
            BOOST_REQUIRE_CLOSE(parsed.getTransformTolerance(), original.getTransformTolerance(), TOLERANCE);
            BOOST_REQUIRE_EQUAL(parsed.getUnknownCostValue(), original.getUnknownCostValue());
            BOOST_REQUIRE_EQUAL(parsed.getUnknownThreshold(), original.getUnknownThreshold());
            BOOST_REQUIRE_CLOSE(parsed.getUpdateFrequency(), original.getUpdateFrequency(), TOLERANCE);      
            BOOST_REQUIRE_EQUAL(parsed.getWidth(), original.getWidth());
            BOOST_REQUIRE_CLOSE(parsed.getzOrigin(), original.getzOrigin(), TOLERANCE);
            BOOST_REQUIRE_CLOSE(parsed.getzResolution(), original.getzResolution(), TOLERANCE);             
            BOOST_REQUIRE_EQUAL(parsed.getzVoxels(), original.getzVoxels());
            
	} catch (std::exception e) {
		BOOST_FAIL(string("Caught exception: ") + e.what());
	}

}

BOOST_AUTO_TEST_SUITE_END()