/*
* This file is part of Bielefeld Type Library (BTL).
*
* Copyright(c) Frederic Siepmann
* http://opensource.cit-ec.de/projects/btl
*
* This file may be licensed under the terms of of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*
* Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman Köster,
* 				Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling,
* 				Denis Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
*/

#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>

#include <iostream>

#include "btl/data/vision1d/LaserData.h"
#include "../../utils.h"

using namespace airobots;
using namespace std;
using namespace xmltio;

static const double TOLERANCE = 0.01;

BOOST_AUTO_TEST_SUITE(LaserDataTest)

BOOST_AUTO_TEST_CASE(testSelfCompatibility) {

	try {

		LaserData original;
		original.setGenerator("test");
		original.setTimestamp(Timestamp(123, 456));
		timeval now;
		gettimeofday(&now, NULL);
		original.setReadTime(now);
		vector<float> points;
		for (unsigned int i = 0; i < 35; i++) {
			points.push_back(((float)rand() / RAND_MAX) * 30.0);
		}
		original.setLaserPoints(points, METER);
		BOOST_REQUIRE_EQUAL(original.getNumLaserPoints(), points.size());

		Location originalLoc(LaserData::getTemplate(), LaserData::getBaseTag());
		originalLoc = original;

		LaserData parsed = extract<LaserData>(originalLoc);

		BOOST_REQUIRE_EQUAL(parsed.getGenerator(), original.getGenerator());
		BOOST_REQUIRE_EQUAL(parsed.getTimestamp(), original.getTimestamp());
		BOOST_REQUIRE_EQUAL(parsed.getReadTime(), original.getReadTime());
		BOOST_REQUIRE_EQUAL(parsed.getReadTimeval().tv_sec, original.getReadTimeval().tv_sec);
		BOOST_REQUIRE_EQUAL(parsed.getReadTimeval().tv_usec, original.getReadTimeval().tv_usec);
		BOOST_REQUIRE_EQUAL(parsed.getReadTimeSeconds(), original.getReadTimeSeconds());
		BOOST_REQUIRE_EQUAL(parsed.getReadTimeMicroSeconds(), original.getReadTimeMicroSeconds());
		BOOST_REQUIRE_EQUAL(parsed.getNumLaserPoints(), original.getNumLaserPoints());
		for (unsigned int i = 0; i < parsed.getNumLaserPoints(); i++) {
			BOOST_REQUIRE_CLOSE(parsed.getLaserPoints(METER)[i], original.getLaserPoints(METER)[i], TOLERANCE);
		}

	} catch (std::exception &e) {
		BOOST_FAIL(string("Caught exception: ") + e.what());
	}

}

BOOST_AUTO_TEST_CASE(testFileCompatibility) {

	Location fileContents(readFile("Laser.xml"), LaserData::getBaseTag());
	LaserData parsed = extract<LaserData>(fileContents);

	BOOST_REQUIRE_EQUAL("test", parsed.getGenerator());
	BOOST_REQUIRE_EQUAL(12345, parsed.getTimestamp().getInserted());
	BOOST_REQUIRE_EQUAL(67890, parsed.getTimestamp().getUpdated());
	BOOST_REQUIRE_EQUAL(MicroTimestamp(123213412, 335435), parsed.getReadTime());
	BOOST_REQUIRE_EQUAL(5, parsed.getNumLaserPoints());
	BOOST_REQUIRE_CLOSE((float) 0.01, parsed.getLaserPoints(METER)[0], 0.00001);
	BOOST_REQUIRE_CLOSE((float) 0.002, parsed.getLaserPoints(METER)[1], 0.00001);
	BOOST_REQUIRE_CLOSE((float) 0.003, parsed.getLaserPoints(METER)[2], 0.00001);
	BOOST_REQUIRE_CLOSE((float) 0.004, parsed.getLaserPoints(METER)[3], 0.00001);
	BOOST_REQUIRE_CLOSE((float) 0.005, parsed.getLaserPoints(METER)[4], 0.00001);

}

BOOST_AUTO_TEST_SUITE_END()
